<?php

namespace HCM\FPA\Controllers;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\FPA\Calculos\Geral\Inicializa;
use HCM\FPA\Services\FeriasService;
use HCM\FPA\Entity\FpagozoferiasEntity;
use HCM\FPA\Entity\FpaferiasEntity;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\Util\Util;

class FeriasController extends Crud {

    protected $dataSource = 'fpa_formferias';
    protected $dataSourceGet = 'getFormFerias';

    protected $parameterBagColumns = array('P_NRORG', 'P_CDOPERADOR');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, entityManager $entityManager, FeriasService $feriasService){
        parent::__construct($dataSourceManager);
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        $this->feriasService = $feriasService;
    }

    public function getFormFerias (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $rows = $dataSet->getRows();
        
        foreach($rows as $key => &$row){
            if($row['DSORDEM'] == 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['DSINICIO']);
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
            }    
            
            if($row['DSORDEM'] == 'LISTA_ESTRUTURA'){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIO']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['DSFIM']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIM']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
            }
            
            if($row['DSORDEM'] === "LISTA_NRESTRUTSIND"){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIO']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                
                if($row['DSFIM']) {
                    $row['LIST_NRESTRUTSIND'] = explode(',',$row['DSFIM']);
                    foreach($row['LIST_NRESTRUTSIND'] as $chave => $value)
                        $row['LIST_NRESTRUTSIND'][$chave] = intval($value);
                        
                    $row['LIST_NMESTRUTSIND'] = explode(",", $row['NMESTRUTSIND']);
                }
            
            }
        }
        $response->addDataSet(new DataSet($this->dataSourceGet, $rows));
    }

    public function setFormFerias (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSource;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if (array_key_exists('NRFORMFERIAS', $row)) {
                    if ($row['NRFORMFERIAS'] === '') {
                        $row['NRFORMFERIAS'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_FORMFERIAS', 1, 12, true);
                    }
                } else {
                    $row['NRFORMFERIAS'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_FORMFERIAS', 1, 12, true);
                }
                
                if(!array_key_exists('DSINICIO', $row)){
                    $row['DSINICIO'] = null;
                }
 
                if(!array_key_exists('DSFIM', $row)){
                    $row['DSFIM'] = null;
                }
                
                // define os campos de descrição de início e fim com base no critério de seleção passado pela row
                if($row['DSORDEM'] === "LISTA_VINCULO"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                }
                
                if($row['DSORDEM'] === "LISTA_ESTRUTURA"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if(is_array($row['DSFIM']) && count($row['DSFIM']) > 0 ){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
                
                if($row['DSORDEM'] === "LISTA_NRESTRUTSIND"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if(is_array($row['DSFIM']) && count($row['DSFIM']) > 0){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
                
                if(!array_key_exists('NRVINCULOM', $row)){
                    $row['NRVINCULOM'] = null;
                }
                if(!array_key_exists('NRFERIAS', $row)){
                    $row['NRFERIAS'] = null;
                }

                $row['DTINIGOZOINI'] = $row['DTINIGOZOFERIAS'];
                $row['DTINIGOZOFIM'] = $row['DTFIMGOZOFERIAS'];
                
                $row['CDOPERADOR'] = $row['CDOPERULTATU'];
                $row['IDEXECUTAPREP'] = 'N';
                
                if(!$row['DTPAGAMENTO']){
                    $row['DTPAGAMENTO'] = substr(DateUtil::subtraiIntervalo($row['DTINIGOZOINI'], 2), 0, 10);
                }
                
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getAprovaFerias(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            set_time_limit(0);
            $filters = Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());
            $parameters = array();
            $ignoraocorrecal = $ignoramodalidcalc = 'S';
            $fieldsUnset = array('P_DTMESCOMPETENC', 'P_NRORG', 'P_NRORG_PADRAO', 'NRESTRUTLEGAL', 'NRESTRUTGEREN', 'NRVINCULOM', 'QTDEDIASRESTANTE', 'NRVINCULOM_LIST', 'NRESTRUTGEREN_LIST', 'NRESTRUTLEGAL_LIST');
            foreach ($filters as $key => $filter) {
                if (array_search($filter['columnName'], $fieldsUnset) !== false){
                    unset($filters[$key]);
                }
                $parameters[$filter['columnName']] = $filter['value'];
            }
            if(array_key_exists('NRVINCULOM_LIST', $parameters) && !empty($parameters['NRVINCULOM_LIST'])){
                $vinculos = is_array($parameters['NRVINCULOM_LIST']) ? implode(',', $parameters['NRVINCULOM_LIST']) : $parameters['NRVINCULOM_LIST'];
                if(array_key_exists('NRVINCULOM', $parameters) && !empty($parameters['NRVINCULOM'])){
                    $parameters['NRVINCULOM'] .= ','.$vinculos; 
                }else{
                    $parameters['NRVINCULOM'] = $vinculos;
                }
                unset($parameters['NRVINCULOM_LIST']);
                unset($vinculos);
            }
            
            if(array_key_exists('NRESTRUTGEREN_LIST', $parameters) && !empty($parameters['NRESTRUTGEREN_LIST'])){
                $parameters['NRESTRUTGEREN'] = $parameters['NRESTRUTGEREN_LIST']; 
                unset($parameters['NRESTRUTGEREN_LIST']);
            }
            
            if(array_key_exists('NRESTRUTLEGAL_LIST', $parameters) && !empty($parameters['NRESTRUTLEGAL_LIST'])){
                $parameters['NRESTRUTLEGAL'] = $parameters['NRESTRUTLEGAL_LIST']; 
                unset($parameters['NRESTRUTLEGAL_LIST']);
            }
            

            $result = $this->feriasService->getAprovaFerias($parameters, $filters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getAprovaFerias', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }
    
    public function deleteFormFerias (DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = $this->dataSource;
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function calculoFerias (DTO\Request\DataSet $request, DTO\Response $response) {
        // Tempo de execucao deve ser ilimitado.
        set_time_limit(0);

        try {
            $dataset = $request->getDataSet()->getRows();
            $this->calculoFolhaService->calculo($dataset[0]);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    public function cadGozoFerias (DTO\Request\DataSet $request, DTO\Response $response) {
        // Tempo de execucao deve ser ilimitado.
        set_time_limit(0);

        try {
            $dataset = $request->getDataSet()->getRows()[0];
            
            // faz com que a data de pagamento de férias seja usada como data de competência para o cálculo
            $dtmescompetenc = DateUtil::getDataDeString(substr($dataset['DTPAGAMENTO'], 0, 10));
            $dtmescompetenc = DateUtil::getPrimeiroDiaDoMes($dtmescompetenc);
            $dtmescompetenc = DateUtil::truncateData($dtmescompetenc);
            
            Inicializa::Container($this->entityManager, $this->hcmParameters, null);
            Inicializa::carregaVariaveis();
            $this->_vinculom = new GpevinculomEntity($this->entityManager, $this->hcmParameters);
            $this->fpaGozoFeriasEntity = new FpagozoferiasEntity($this->entityManager, $this->hcmParameters);
        
            
            $vinculos = $this->_vinculom->retornaVinculosAtivosMesCompetencia(
            $this->hcmParameters->getNrorgTrabalho(), $dtmescompetenc, $dataset['DSINICIO'], $dataset['DSFIM'], $dataset['DSORDEM'], true);
            
            $dtIniGozo = isset($dataset['DTINIGOZOFERIAS']) ? $dataset['DTINIGOZOFERIAS'] : $dataset['DTINIGOZOINI'];
            $dtFimGozo = isset($dataset['DTFIMGOZOFERIAS']) ? $dataset['DTFIMGOZOFERIAS'] : $dataset['DTINIGOZOFIM'];
            $nrDiasFerias = isset($dataset['QTDIAGOZOFERIAS']) ? $dataset['QTDIAGOZOFERIAS'] : null;
            
            $idadiant13salar = isset($dataset["IDADIANT13SALAR"]) ? $dataset["IDADIANT13SALAR"] : "N";
            $diasabono = isset($dataset['QTDIASABONOFERI']) ? $dataset['QTDIASABONOFERI'] : null;
            
            $dtAviso = isset($dataset['DTAVISOFERIAS']) ? DateUtil::getDataDeString($dataset["DTAVISOFERIAS"],DateUtil::FORMATO_BRASILEIRO,true): null;

            $nrferias = isset($dataset['NRFERIAS']) ? $dataset['NRFERIAS'] : null;
            $dsgozoferias = isset($dataset['DSGOZOFERIAS']) ? $dataset['DSGOZOFERIAS'] : null;
            
            $idnaopagaadic = isset($dataset['IDNAOPAGAADIC']) ? $dataset['IDNAOPAGAADIC'] : "N";
            
            $msg = $this->fpaGozoFeriasEntity->persisteGozoFeriasAutomatico($vinculos, $dtIniGozo, $dtFimGozo, $idadiant13salar, $diasabono, $dsgozoferias, $nrferias, $dtAviso, $nrDiasFerias, $idnaopagaadic);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("msg", $msg));
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    public function aprovaFerias (DTO\Request\DataSet $request, DTO\Response $response) {
        // Tempo de execucao deve ser ilimitado.
        set_time_limit(0);
        
        try {
            $this->fpaGozoFeriasEntity = new FpagozoferiasEntity($this->entityManager, $this->hcmParameters);
            $this->fpaFeriasEntity = new FpaferiasEntity($this->entityManager, $this->hcmParameters);
            
            $dataset = $request->getDataSet()->getRows()[0];
            $vinculosNaoInseridos = array();
            foreach($dataset as $key => $row){
                $ferias = $row['NRFERIAS'];
                $vinculo = $row['NRVINCULOM']; 
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $criouGozo = false;
                if(isset($row['DTINIPREVISAO']) && isset($row['DTFINPREVISAO'])){
                    $dtIniGozo = $row['DTINIPREVISAO'];
                    $dtFimGozo = $row['DTFINPREVISAO'];
                    $dtRetornoFerias = DateUtil::adicionaIntervalo(DateUtil::getDataDeString(substr($dtFimGozo,0,10), DateUtil::FORMATO_BRASILEIRO, true),1);
                    $qtdDiasAbono = isset($row['QTABONOPREVISAO']) ? $row['QTABONOPREVISAO'] : null; 
                    $this->fpaGozoFeriasEntity->incluiGozoFerias($nrorg, $ferias, $dtIniGozo, $dtFimGozo, $qtdDiasAbono, $dtRetornoFerias,'N');
                    $criouGozo = true;
                }
                if(isset($row['DTINIPREVISAO2']) && isset($row['DTFINPREVISAO2'])){
                    $dtIniGozo = $row['DTINIPREVISAO2'];
                    $dtFimGozo = $row['DTFINPREVISAO2'];
                    $dtRetornoFerias = DateUtil::adicionaIntervalo(DateUtil::getDataDeString(substr($dtFimGozo,0,10), DateUtil::FORMATO_BRASILEIRO, true),1);
                    $qtdDiasAbono = isset($row['QTABONOPREVISAO2']) ? $row['QTABONOPREVISAO2'] : null; 
                    $this->fpaGozoFeriasEntity->incluiGozoFerias($nrorg, $ferias, $dtIniGozo, $dtFimGozo, $qtdDiasAbono, $dtRetornoFerias,'N');
                    $criouGozo = true;
                }
                if(isset($row['DTINIPREVISAO3']) && isset($row['DTFINPREVISAO3'])){
                    $dtIniGozo = $row['DTINIPREVISAO3'];
                    $dtFimGozo = $row['DTFINPREVISAO3'];
                    $dtRetornoFerias = DateUtil::adicionaIntervalo(DateUtil::getDataDeString(substr($dtFimGozo,0,10), DateUtil::FORMATO_BRASILEIRO, true),1);
                    $qtdDiasAbono = isset($row['QTABONOPREVISAO3']) ? $row['QTABONOPREVISAO3'] : null; 
                    $this->fpaGozoFeriasEntity->incluiGozoFerias($nrorg, $ferias, $dtIniGozo, $dtFimGozo, $qtdDiasAbono, $dtRetornoFerias,'N');
                    $criouGozo = true;
                }
                
                if($criouGozo){
                    $this->fpaFeriasEntity->resetaFeriasPrevista($nrorg,$ferias);
                }else {
                    $vinculosNaoInseridos[]=$vinculo;
                }
            }
             
            if(!empty($vinculosNaoInseridos)){
                $mensagem = "Os gozos de férias dos vínculos de Nº ".implode(', ', $vinculosNaoInseridos)." não foram inseridos pois as datas de início/fim de previsão não foram devidamente preenchidas";
                $response->addNotification(new Notification($mensagem, Notification::TYPE_ALERT));
            }else {
                $response->addNotification(new Notification("Gozos de férias registrados com sucesso!", Notification::TYPE_SUCCESS));
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    public function reprovaFerias (DTO\Request\DataSet $request, DTO\Response $response){
        try{
            set_time_limit(0);
            $dataset = $request->getDataSet()->getRows()[0];
            foreach($dataset as $key => $row){
                $ferias = $row['NRFERIAS'];
                $vinculo = $row['NMVINCULOM'];
                $estrutGeren = $row['NRESTRUTGEREN'];
                $nmgerem = $row['NMESTRUTGEREN'];
                $dtinicioprev1 = $row['DTINIPREVISAO'] ? substr($row['DTINIPREVISAO'],0,10) : 'não informado';
                $diasprev1 = $row['QTDIASPREVISAO'] ? substr($row['QTDIASPREVISAO'],0,10) : 'não informado';
                $dtfimprev1 = $row['DTFINPREVISAO'] ? substr($row['DTFINPREVISAO'],0,10) : 'não informado';
                $mensagem = $row['DSMENSAGEM'] ? $row['DSMENSAGEM'] : '';
                $result = $this->feriasService->enviaEmailReprova($vinculo, $estrutGeren, $nmgerem, $dtinicioprev1, $diasprev1, $dtfimprev1, $mensagem);
            }
              
            if(gettype($result) == 'string'){
                $response->addNotification(new \Zeedhi\Framework\DTO\Response\Notification($result, \Zeedhi\Framework\DTO\Response\Notification::TYPE_ALERT));
            }else{
                $response->addNotification(new \Zeedhi\Framework\DTO\Response\Notification('Success', \Zeedhi\Framework\DTO\Response\Notification::TYPE_SUCCESS));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retGozoFerias (DTO\Request\DataSet $request, DTO\Response $response) {
        // Tempo de execucao deve ser ilimitado.
        set_time_limit(0);

        try {
            // faz com que a data de pagamento de férias seja usada como data de competência para o cálculo
            $row = $request->getDataSet()->getRows()[0];
            $dtmescompetenc = DateUtil::getDataDeString(substr($row['DTPAGAMENTO'], 0, 10));
            $dtmescompetenc = DateUtil::getPrimeiroDiaDoMes($dtmescompetenc);
            $dtmescompetenc = DateUtil::truncateData($dtmescompetenc);
            
            Inicializa::Container($this->entityManager, $this->hcmParameters, null);
            Inicializa::carregaVariaveis();
            $this->_vinculom = new GpevinculomEntity($this->entityManager, $this->hcmParameters);
            $this->fpaGozoFeriasEntity = new FpagozoferiasEntity($this->entityManager, $this->hcmParameters);
        

            $vinculos = $this->_vinculom->retornaVinculosAtivosMesCompetencia(
            $this->hcmParameters->getNrorgTrabalho(), $dtmescompetenc, $row['DSINICIO'], $row['DSFIM'], $row['DSORDEM'], true);
            
            $dtIniGozo = isset($row['DTINIGOZOFERIAS']) ? $row['DTINIGOZOFERIAS'] : $row['DTINIGOZOINI'];
            $dtFimGozo = isset($row['DTFIMGOZOFERIAS']) ? $row['DTFIMGOZOFERIAS'] : $row['DTINIGOZOFIM'];
            
            $idadiant13salar = 'N';
            $diasabono = isset($row['QTDIASABONOFERI']) ? $row['QTDIASABONOFERI'] : null;
            
            $dtAviso = isset($row['DTAVISOFERIAS']) ? DateUtil::getDataDeString($row["DTAVISOFERIAS"],DateUtil::FORMATO_BRASILEIRO,true): null;
            
            $nrDiasFerias = isset($row['QTDIAGOZOFERIAS']) ? $row['QTDIAGOZOFERIAS'] : null;
            
            $msg = $this->fpaGozoFeriasEntity->retrocedeGozoFeriasAutomatico($vinculos, $dtIniGozo, $dtFimGozo);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("msg", $msg));
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}