<?php
define('HCM_ERROR_REPORTING', (E_ALL | E_STRICT) ^ (E_WARNING | E_NOTICE));
require_once __DIR__ . '/../../backend/bootstrap.php';

//if (!defined("OBJ_DESCRIPTION_PATH"))define("OBJ_DESCRIPTION_PATH", APP_PATH."/files/esocial/obj_description");
//if (!defined("TEMPLATES_PATH"))define("TEMPLATES_PATH", __DIR__."\\..\\..\\files\\esocial\\templates");

function main()
{
    $argv = $_SERVER["argv"];
    $versao = '2.3';
    echo "Gerando $argv[1] para versao $versao\n";
    if ($argv[1] == null) {
        echo "[AVISO] Favor informar o nome do evento apos o comando ou ALL para todos\n";
        die;
    }
    else {
        $nmevento = $argv[1];

        // Busca template base para geração dos templates dos eventos
        $templateDir = __DIR__ . "/../../backend/files/esocial/templates/xml";
        if (!file_exists($templateDir."/template-xml-esocial.twig")) {
            echo "[ERRO] Falta arquivo template-xml-esocial.twig na pasta $templateDir\n";
            die;
        }

        // Busca o(s) evento(s) para fazer a geração
        $eventos = GpeEsocial::getEventos($nmevento, $versao);
        $cont = 0;
        foreach ($eventos as $evento) {
            echo "[INFO] Geração do template {$evento['NMEVENTO']}\n";
            $nomeEvento = $evento["NMEVENTO"];
            $versaoEvento = $evento["DSVERSAO"];
            $dadosEvento = GpeEsocial::getDadosEvento($evento["NREVENTO"]);

            $descricaoBuilder = new DescricaoDoArquivo (
                $nomeEvento, $versaoEvento, $dadosEvento
            );
            $descricaoDoEvento = $descricaoBuilder->buildData();

            $xmlBuilder = new Template($descricaoDoEvento, $templateDir);
            $template = $xmlBuilder->build("template-xml-esocial.twig");

            $diretorioVersao = str_replace(".", "-", $versaoEvento);
            if (!is_dir($templateDir . "/v_$diretorioVersao")) {
                mkdir($templateDir . "/v_$diretorioVersao");
                chmod($templateDir . "/v_$diretorioVersao", "777");
                echo "[INFO] Criado diretorio " . $templateDir . "/v_$diretorioVersao\n";
            }

            if (file_exists($templateDir."/v_$diretorioVersao/$nomeEvento.xml.twig")) {
                echo "[AVISO] Ja existia o arquivo $nomeEvento.xml.twig que foi substituido por um novo\n";
            }
            file_put_contents($templateDir."/v_$diretorioVersao/$nomeEvento.xml.twig", formatXML($template));
            $cont++;
        }
        echo "Criado(s) $cont template(s) twig\n";
    }
}

function formatXML($xml)
{
    // Este bloco transforma o texto gerado, para que as tags do twig sejam ocultadas, a fim de possibilitar a 
    // formatacao do documentos, que somente e disponivel para um texto no formato .xml
    $xml = str_replace("{{", "aaa{{", $xml);
    $xml = str_replace("}}", "}}aaa", $xml);
    $xml = str_replace("{%", "<TEMPORARY_TAG>aaa{%", $xml);
    $xml = str_replace("%}", "%}aaa</TEMPORARY_TAG>", $xml);

    $xml    = new SimpleXMLElement($xml);
    $domxml = new DOMDocument('1.0');
    $domxml->preserveWhiteSpace = false;
    $domxml->formatOutput = true;
    $domxml->loadXML($xml->asXML());

    // Remocao do texto temporario gerado.
    $xml =  $domxml->saveXML();
    $xml = str_replace("aaa{{", "{{", $xml);
    $xml = str_replace("}}aaa", "}}", $xml);
    $xml = str_replace("<TEMPORARY_TAG>aaa{%", "{%", $xml);
    $xml = str_replace("%}aaa</TEMPORARY_TAG>", "%}", $xml);

    // Remove a primeira linha "<?xml version='1.0'>" pois não é utilizada pelo esocial ao enviar o evento dentro lote
    $xml = preg_replace('!^[^>]+>(\r\n|\n)!','',$xml);

    // Altera namespace para xmlns
    $xml = str_replace("namespace", "xmlns", $xml);

    return $xml;
}

class GpeEsocial
{
    public static function getEventos($evento, $versao)
    {
        $sql = "select *
                  from eso_evento
                 where idativo = 'S'
                   and ((nmevento = '$evento') or (upper('$evento') = 'ALL'))
                   and dsversao = '$versao'
                 order by nmevento";

        return \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql);
    }

    public static function getDadosEvento($nrevento)
    {
        $sql = "select eso_registro.nrregistro,
                       eso_registro.nrcampo, eso_registro.nmcampo,
                       eso_registro.nrcampopai, eso_registro.nmcampopai,
                       eso_registro.dselemento, eso_registro.dstipo , eso_registro.dsocorrencia, eso_registro.nrtamanho,
                       eso_registro.nrdecimais, eso_registro.dsregistro
              from eso_registro
             where eso_registro.nrevento = :NREVENTO
             order by eso_registro.nrregistro";
        $sql = "SELECT teste.*, rownum from ($sql) teste";

        return \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, array(":NREVENTO" => $nrevento));
    }
}

class DescricaoDoArquivo
{

    private $arquivo;
    private $versao;
    private $elementos;

    private $raiz;
    private $namespace;

    private $builtObj;

    const ATRIBUTO = "A";
    const ELEMENTO = "E";
    const GRUPO    = "G";

    /**
     * Metodo construtor.
     *
     * @param $arquivo   String Nome do arquivo especificado do arquivo de integracao (ex.: S-1100, para o e-Social).
     * @param $versao    String Versão do arquivo, em relação ao manual onde foi especificado (ex.: 1.1).
     * @param $elementos array  Array com os dados dos elementos que compõe o arquivo. Cada elemento deste array deverá
     *                          apresentar a seguinte estrutura:
     *                              "NRCAMPO"    : "<numero_do_elemento>",
     *                              "NMCAMPO"    : "<nome_do_elemento>",
     *                              "NRCAMPOPAI" : "<numero_do_elemento_pai>",
     *                              "NMCAMPOPAI" : "<nome_do_elemento_pai>",
     *                              "ELEMENTO"   : "<G,E,A (grupo, elemento ou atributo)>",
     *                              "TIPO"       : "<null, D, C, N (char, number, date)>",
     *                              "OCORRENCIA" : "<ocorrencia_do_elemento> (ex.: 1-1)",
     *                              "TAMANHO"    : "<tamanho_do_elemento>",
     *                              "DECIMAIS"   : "<numero_de_casas_decimais_do_elemento>",
     *                              "DESCRICAO"  : "<descricao_do_elemento>",
     * @param $raiz      String Especifica o nome do elemento raiz do arquivo. Caso nada seja passado, o builder deverá
     *                          pesquisar dentre os elementos aquele que nao possui o atributo "pai" e assumir este como
     *                          a raiz do arquivo.
     *
     * @return void
     */
    public function __construct($arquivo, $versao, $elementos = array(), $raiz = null)
    {
        $this->arquivo   = $arquivo;
        $this->versao    = $versao;
        $this->elementos = $elementos;
        $this->raiz      = $raiz;
        $this->namespace = null;
    }


    public function buildData()
    {
        $dataCriacaoArquivo = new \DateTime();

        $this->builtObj["__raiz"]     = $this->raiz ? : $this->procurarElementoRaiz();
        $this->builtObj["__versao"]   = $this->versao;
        $this->builtObj["__arquivo"]  = $this->arquivo;
        $this->builtObj["__dataHora"] = $dataCriacaoArquivo->format("d/m/Y H:i:s");
        $this->buildComponent();
        $this->setElementosFilhos();
        $this->setXmlNamespace();
        //var_dump(end($this->builtObj));die;
        //var_dump($this->builtObj[$this->builtObj["__raiz"]]);die;
        return $this->builtObj;
    }

    private function setElementosFilhos()
    {
        foreach ($this->elementos as $elemento) {
            if ($elemento["NRCAMPOPAI"]) {
                if ($elemento["DSELEMENTO"] === self::ATRIBUTO) {
                    if (!in_array($elemento["NRCAMPO"], $this->builtObj[$elemento["NRCAMPOPAI"]]["atributos"])) {
                        $this->builtObj[$elemento["NRCAMPOPAI"]]["atributos"][] = $elemento["NRCAMPO"];
                    }
                } else {
                    if (!in_array($elemento["NRCAMPO"], $this->builtObj[$elemento["NRCAMPOPAI"]]["filhos"])) {
                        $this->builtObj[$elemento["NRCAMPOPAI"]]["filhos"][] = $elemento["NRCAMPO"];
                    }
                }
            }
        }
    }

    private function setXmlNamespace()
    {
        $nrcampo = count($this->elementos) + 1;
        $arrVersao = explode('.', $this->builtObj["__versao"]);
        $versao = 'v' .
            str_pad($arrVersao[0], 2, '0', STR_PAD_LEFT) . '_' .
            str_pad($arrVersao[1], 2, '0', STR_PAD_LEFT) . '_' .
            str_pad(isset($arrVersao[3]) ? $arrVersao[3] : '0', 2, '0', STR_PAD_LEFT)
        ;
        $this->builtObj[$nrcampo] = array (
            "nrElemento"    => $nrcampo,
            "nmElemento"    => 'namespace',
            "vrElemento"    => 'http://www.esocial.gov.br/schema/evt/' . $this->namespace . '/' . $versao,
            "nrElementoPai" => $this->builtObj["__raiz"],
            "nmElementoPai" => $this->builtObj[$this->builtObj["__raiz"]]['nmElemento'],
            "tipo"          => "C",
            "elemento"      => "A",
            "tamanho"       => 100,
            "ocorrencia"    => $this->getOcorrencias("1-1"),
            "decimais"      => null,
            "validacoes"    => array(),
            "atributos"     => array(),
            "filhos"        => array(),
        );
        $this->builtObj[$this->builtObj["__raiz"]]["atributos"][] = $nrcampo;
    }

    private function buildComponent()
    {
        foreach ($this->elementos as $elemento) {
            $this->builtObj[$elemento["NRCAMPO"]] = array (
                "nrElemento"    => $elemento["NRCAMPO"],
                "nmElemento"    => $elemento["NMCAMPO"],
                "nrElementoPai" => $elemento["NRCAMPOPAI"],
                "nmElementoPai" => $elemento["NMCAMPOPAI"],
                "tipo"          => $elemento["DSTIPO"],
                "elemento"      => $elemento["DSELEMENTO"],
                "tamanho"       => $elemento["NRTAMANHO"],
                "ocorrencia"    => $this->getOcorrencias($elemento["DSOCORRENCIA"]),
                "decimais"      => $elemento["NRDECIMAIS"],
                "validacoes"    => $this->getValidacoes($elemento["DSREGISTRO"]),
                "atributos"     => array(),
                "filhos"        => array(),
            );
            if ($elemento['NRCAMPOPAI'] == $this->builtObj["__raiz"]) {
                $this->namespace = $elemento["NMCAMPO"];
            }
        }
    }

    private function getOcorrencias($str)
    {
        list($menor, $maior) = explode("-", $str);
        return array(
            "menor" => $menor,
            "maior" => $maior,
        );
    }

    private function getValidacoes($descricao)
    {
        preg_match_all("/REGRA_\w+/", $descricao, $result);
        return $result[0];
    }

    private function procurarElementoRaiz()
    {
        foreach ($this->elementos as $elemento) {
            if ((bool)$elemento["NRCAMPOPAI"] === false) {
                return $elemento["NRCAMPO"];
            }
        }

        return null;
    }

}

class Template
{

    private $descricaoDoObjeto;
    private $templateDir;

    function __construct($descricaoDoObjeto, $templateDir) {
        $this->descricaoDoObjeto = $descricaoDoObjeto;
        $this->templateDir       = $templateDir;
    }

    public function build($template)
    {
        \Twig_Autoloader::register();
        $loader   = new \Twig_Loader_Filesystem($this->templateDir);
        $twig     = new \Twig_Environment($loader, array());
        $template = $twig->loadTemplate($template);
        $file     = $template->render(array("arquivo" => $this->descricaoDoObjeto));
        return $file;
    }

}

main();